@@ -42,6 +42,9 @@
 --   * "Diagrams.TwoD.Shapes" defines other two-dimensional shapes,
 --     e.g. various polygons.
 --
+--   * "Diagrams.TwoD.Arrow" contains tools for drawing arrows between
+--     things.
+--
 --   * "Diagrams.TwoD.Text" defines primitive text diagrams.
 --
 --   * "Diagrams.TwoD.Image" allows importing external images into diagrams.
@@ -58,27 +61,25 @@
 -----------------------------------------------------------------------------
 module Diagrams.TwoD
        ( -- * R^2
-         R2, r2, unr2
-       , P2, p2, unp2
+         R2, r2, unr2, mkR2
+       , P2, p2, unp2, mkP2
        , T2
        , unitX, unitY, unit_X, unit_Y
-       , direction, fromDirection, e
+       , direction, fromDirection
 
          -- * Angles
        , tau
-       , Angle(..)
-       , Turn(..), CircleFrac, Rad(..), Deg(..)
-       , fullCircle, convertAngle
 
          -- * Paths
          -- ** Stroking
-       , stroke, stroke', strokeT, strokeT', strokeLine, strokeLoop
-       , strokeLocT, strokeLocLine, strokeLocLoop
+       , stroke, stroke', strokeTrail, strokeT, strokeTrail', strokeT'
+       , strokeLine, strokeLoop
+       , strokeLocTrail, strokeLocT, strokeLocLine, strokeLocLoop
        , FillRule(..), fillRule
-       , StrokeOpts(..)
+       , StrokeOpts(..), vertexNames, queryFillRule
 
          -- ** Clipping
-       , clipBy
+       , clipBy, clipTo, clipped
 
          -- * Shapes
          -- ** Rules
@@ -93,10 +94,13 @@ module Diagrams.TwoD
        , arc'
        , arcCW
        , wedge
+       , arcBetween
+       , annularWedge
 
          -- ** General polygons
        , polygon, polyTrail
-       , PolygonOpts(..), PolyType(..), PolyOrientation(..)
+       , PolygonOpts(..), polyType, polyOrient, polyCenter
+       , PolyType(..), PolyOrientation(..)
 
          -- ** Star polygons
        , StarOpts(..), star
@@ -108,6 +112,7 @@ module Diagrams.TwoD
        , square
        , pentagon
        , hexagon
+       , heptagon
        , septagon
        , octagon
        , nonagon
@@ -121,14 +126,51 @@ module Diagrams.TwoD
 
          -- ** Other shapes
        , roundedRect, roundedRect'
-       , RoundedRectOpts(..)
+       , RoundedRectOpts(..), radiusTL, radiusTR, radiusBL, radiusBR
+
+         -- ** Arrows
+       , arrowV, arrowV'
+       , arrowAt, arrowAt'
+       , arrowBetween, arrowBetween'
+       , connect, connect'
+       , connectPerim, connectPerim'
+       , connectOutside, connectOutside'
+       , arrow, arrow'
+       , straightShaft
+       , module Diagrams.TwoD.Arrowheads
+
+       , ArrowOpts(..)
+
+       , arrowHead
+       , arrowTail
+       , arrowShaft
+       , headGap
+       , tailGap
+       , gaps, gap
+       , headTexture
+       , headStyle
+       , tailTexture
+       , tailStyle
+       , shaftTexture
+       , shaftStyle
+       , headLength
+       , tailLength
+       , lengths
 
          -- * Text
        , text, topLeftText, alignedText, baselineText
-       , font, fontSize, italic, oblique, bold
+       , font, italic, oblique, bold, fontSize
+       , fontSizeO, fontSizeL, fontSizeN, fontSizeG
 
          -- * Images
+       , DImage(..), ImageData(..)
+       , Embedded, External
        , image
+       , loadImageEmb
+       , loadImageExt
+       , uncheckedImageRef
+       , raster
+       , rasterDia
 
          -- * Transformations
          -- ** Rotation
@@ -152,6 +194,10 @@ module Diagrams.TwoD
        , shearingX, shearX
        , shearingY, shearY
 
+         -- * Deformations - non-affine transforms
+       , parallelX0, perspectiveX1, parallelY0, perspectiveY1
+       , facingX, facingY
+
          -- * Combinators
          -- ** Combining multiple diagrams
        , (===), (|||), atAngle
@@ -175,6 +221,11 @@ module Diagrams.TwoD
        , alignX, alignY
        , centerX, centerY, centerXY
 
+         -- * Snugging
+       , snugL, snugR, snugT, snugB, snugTL, snugTR, snugBL, snugBR
+       , snugX, snugY
+       , snugCenterX, snugCenterY, snugCenterXY
+
          -- * Size
          -- ** Computing size
        , width, height, size2D, sizeSpec2D
@@ -187,17 +238,48 @@ module Diagrams.TwoD
          -- ** Adjusting size
        , sized, sizedAs
 
+        -- * Textures
+      , Texture(..), solid 
+      , SpreadMethod(..), GradientStop(..), mkStops, getFillTexture
+      , fillTexture, getLineTexture, lineTexture, lineTextureA
+      , stopFraction, stopColor
+
+      , LGradient(..), lGradStops, lGradTrans, lGradStart, lGradEnd
+      , lGradSpreadMethod, defaultLG, _LG, mkLinearGradient
+
+      , RGradient(..)
+      , rGradStops, rGradCenter0, rGradRadius0, rGradCenter1, rGradRadius1
+      , rGradTrans, rGradSpreadMethod, defaultRG, _RG, mkRadialGradient
+
+        -- ** Colors
+      , fillColor, fc, fcA, recommendFillColor
+      , lineColor, lc, lcA, _SC
+
+       -- ** Width
+       , LineWidth, getLineWidth, lineWidth, lineWidthA
+       , lw, lwN, lwO, lwL, lwG
+       , ultraThin, veryThin, thin, medium, thick, veryThick, ultraThick, none
+       , tiny, verySmall, small, normal, large, veryLarge, huge
+
+       -- ** Dashing
+       , Dashing(..), DashingA, getDashing
+       , dashing, dashingO, dashingL, dashingN, dashingG
+
          -- * Visual aids for understanding the internal model
        , showOrigin
        , showOrigin'
-       , OriginOpts(..)
+       , OriginOpts(..), oColor, oScale, oMinSize
        , showLabels
 
        ) where
 
 import           Diagrams.TwoD.Align
 import           Diagrams.TwoD.Arc
+import           Diagrams.TwoD.Arrow
+import           Diagrams.TwoD.Arrowheads
+import           Diagrams.TwoD.Attributes
 import           Diagrams.TwoD.Combinators
+import           Diagrams.TwoD.Deform
 import           Diagrams.TwoD.Ellipse
 import           Diagrams.TwoD.Image
 import           Diagrams.TwoD.Model
