@@ -1,4 +1,5 @@
 {-# LANGUAGE FlexibleContexts #-}
+{-# LANGUAGE TemplateHaskell  #-}
 {-# LANGUAGE TypeFamilies     #-}
 
 -----------------------------------------------------------------------------
@@ -25,6 +26,7 @@ module Diagrams.TwoD.Shapes
        , square
        , pentagon
        , hexagon
+       , heptagon
        , septagon
        , octagon
        , nonagon
@@ -39,13 +41,13 @@ module Diagrams.TwoD.Shapes
          -- * Other shapes
 
        , roundedRect
-       , RoundedRectOpts(..)
+       , RoundedRectOpts(..), radiusTL, radiusTR, radiusBL, radiusBR
        , roundedRect'
        ) where
 
 import           Diagrams.Core
 
-import           Diagrams.Coordinates
+import           Diagrams.Angle
 import           Diagrams.Located        (at)
 import           Diagrams.Path
 import           Diagrams.Segment
@@ -58,53 +60,54 @@ import           Diagrams.TwoD.Types
 
 import           Diagrams.Util
 
+import           Control.Lens            (makeLenses, op, (&), (.~), (^.))
 import           Data.Default.Class
 import           Data.Semigroup
 
 -- | Create a centered horizontal (L-R) line of the given length.
 --
---   <<diagrams/hruleEx.svg#diagram=hruleEx&width=300>>
+--   <<diagrams/src_Diagrams_TwoD_Shapes_hruleEx.svg#diagram=hruleEx&width=300>>
 --
---   > hruleEx = vcat' with {sep = 0.2} (map hrule [1..5])
+--   > hruleEx = vcat' (with & sep .~ 0.2) (map hrule [1..5])
 --   >         # centerXY # pad 1.1
 hrule :: (TrailLike t, V t ~ R2) => Double -> t
-hrule d = trailLike $ trailFromSegments [straight (d & 0)] `at` (p2 (-d/2,0))
+hrule d = trailLike $ trailFromSegments [straight $ r2 (d, 0)] `at` (p2 (-d/2,0))
 
 -- | Create a centered vertical (T-B) line of the given length.
 --
---   <<diagrams/vruleEx.svg#diagram=vruleEx&height=100>>
+--   <<diagrams/src_Diagrams_TwoD_Shapes_vruleEx.svg#diagram=vruleEx&height=100>>
 --
---   > vruleEx = hcat' with {sep = 0.2} (map vrule [1, 1.2 .. 2])
+--   > vruleEx = hcat' (with & sep .~ 0.2) (map vrule [1, 1.2 .. 2])
 --   >         # centerXY # pad 1.1
 vrule :: (TrailLike t, V t ~ R2) => Double -> t
-vrule d = trailLike $ trailFromSegments [straight (0 & (-d))] `at` (p2 (0,d/2))
+vrule d = trailLike $ trailFromSegments [straight $ r2 (0, (-d))] `at` (p2 (0,d/2))
 
 -- | A square with its center at the origin and sides of length 1,
 --   oriented parallel to the axes.
 --
---   <<diagrams/unitSquareEx.svg#diagram=unitSquareEx&width=100>>
+--   <<diagrams/src_Diagrams_TwoD_Shapes_unitSquareEx.svg#diagram=unitSquareEx&width=100>>
 unitSquare :: (TrailLike t, V t ~ R2) => t
-unitSquare = polygon with { polyType   = PolyRegular 4 (sqrt 2 / 2)
-                          , polyOrient = OrientH }
+unitSquare = polygon (def & polyType   .~ PolyRegular 4 (sqrt 2 / 2)
+                          & polyOrient .~ OrientH)
 
 -- > unitSquareEx = unitSquare # pad 1.1 # showOrigin
 
 -- | A square with its center at the origin and sides of the given
 --   length, oriented parallel to the axes.
 --
---   <<diagrams/squareEx.svg#diagram=squareEx&width=200>>
+--   <<diagrams/src_Diagrams_TwoD_Shapes_squareEx.svg#diagram=squareEx&width=200>>
 square :: (TrailLike t, Transformable t, V t ~ R2) => Double -> t
 square d = rect d d
 
--- > squareEx = hcat' with {sep = 0.5} [square 1, square 2, square 3]
--- >          # centerXY # pad 1.1 # lw 0.03
+-- > squareEx = hcat' (with & sep .~ 0.5) [square 1, square 2, square 3]
+-- >          # centerXY # pad 1.1
 
 -- | @rect w h@ is an axis-aligned rectangle of width @w@ and height
 --   @h@, centered at the origin.
 --
---   <<diagrams/rectEx.svg#diagram=rectEx&width=150>>
+--   <<diagrams/src_Diagrams_TwoD_Shapes_rectEx.svg#diagram=rectEx&width=150>>
 rect :: (TrailLike t, Transformable t, V t ~ R2) => Double -> Double -> t
-rect w h = trailLike . head . pathTrails $ unitSquare # scaleX w # scaleY h
+rect w h = trailLike . head . op Path $ unitSquare # scaleX w # scaleY h
 
 -- > rectEx = rect 1 0.7 # pad 1.1
 
@@ -136,18 +139,18 @@ rect w h = trailLike . head . pathTrails $ unitSquare # scaleX w # scaleY h
 --
 --   The polygon will be oriented with one edge parallel to the x-axis.
 regPoly :: (TrailLike t, V t ~ R2) => Int -> Double -> t
-regPoly n l = polygon with { polyType =
+regPoly n l = polygon (def & polyType .~
                                PolySides
-                                 (repeat (1/ fromIntegral n :: Turn))
+                                 (repeat (1/fromIntegral n @@ turn))
                                  (replicate (n-1) l)
-                           , polyOrient = OrientH
-                           }
+                           & polyOrient .~ OrientH
+                           )
 
 -- > shapeEx sh   = sh 1 # pad 1.1
 -- > triangleEx   = shapeEx triangle
 -- > pentagonEx   = shapeEx pentagon
 -- > hexagonEx    = shapeEx hexagon
--- > septagonEx   = shapeEx septagon
+-- > heptagonEx   = shapeEx heptagon
 -- > octagonEx    = shapeEx octagon
 -- > nonagonEx    = shapeEx nonagon
 -- > decagonEx    = shapeEx decagon
@@ -161,69 +164,85 @@ eqTriangle = triangle
 -- | An equilateral triangle, with sides of the given length and base
 --   parallel to the x-axis.
 --
---   <<diagrams/triangleEx.svg#diagram=triangleEx&width=100>>
+--   <<diagrams/src_Diagrams_TwoD_Shapes_triangleEx.svg#diagram=triangleEx&width=100>>
 triangle :: (TrailLike t, V t ~ R2) => Double -> t
 triangle = regPoly 3
 
 -- | A regular pentagon, with sides of the given length and base
 --   parallel to the x-axis.
 --
---   <<diagrams/pentagonEx.svg#diagram=pentagonEx&width=100>>
+--   <<diagrams/src_Diagrams_TwoD_Shapes_pentagonEx.svg#diagram=pentagonEx&width=100>>
 pentagon :: (TrailLike t, V t ~ R2) => Double -> t
 pentagon = regPoly 5
 
 -- | A regular hexagon, with sides of the given length and base
 --   parallel to the x-axis.
 --
---   <<diagrams/hexagonEx.svg#diagram=hexagonEx&width=100>>
+--   <<diagrams/src_Diagrams_TwoD_Shapes_hexagonEx.svg#diagram=hexagonEx&width=100>>
 hexagon :: (TrailLike t, V t ~ R2) => Double -> t
 hexagon = regPoly 6
 
--- | A regular septagon, with sides of the given length and base
+-- | A regular heptagon, with sides of the given length and base
 --   parallel to the x-axis.
 --
---   <<diagrams/septagonEx.svg#diagram=septagonEx&width=100>>
+--   <<diagrams/src_Diagrams_TwoD_Shapes_heptagonEx.svg#diagram=heptagonEx&width=100>>
+heptagon :: (TrailLike t, V t ~ R2) => Double -> t
+heptagon = regPoly 7
+
+-- | A synonym for 'heptagon'.  It is, however, completely inferior,
+--   being a base admixture of the Latin /septum/ (seven) and the
+--   Greek γωνία (angle).
 septagon :: (TrailLike t, V t ~ R2) => Double -> t
-septagon = regPoly 7
+septagon = heptagon
 
 -- | A regular octagon, with sides of the given length and base
 --   parallel to the x-axis.
 --
---   <<diagrams/octagonEx.svg#diagram=octagonEx&width=100>>
+--   <<diagrams/src_Diagrams_TwoD_Shapes_octagonEx.svg#diagram=octagonEx&width=100>>
 octagon :: (TrailLike t, V t ~ R2) => Double -> t
 octagon = regPoly 8
 
 -- | A regular nonagon, with sides of the given length and base
 --   parallel to the x-axis.
 --
---   <<diagrams/nonagonEx.svg#diagram=nonagonEx&width=100>>
+--   <<diagrams/src_Diagrams_TwoD_Shapes_nonagonEx.svg#diagram=nonagonEx&width=100>>
 nonagon :: (TrailLike t, V t ~ R2) => Double -> t
 nonagon = regPoly 9
 
 -- | A regular decagon, with sides of the given length and base
 --   parallel to the x-axis.
 --
---   <<diagrams/decagonEx.svg#diagram=decagonEx&width=100>>
+--   <<diagrams/src_Diagrams_TwoD_Shapes_decagonEx.svg#diagram=decagonEx&width=100>>
 decagon :: (TrailLike t, V t ~ R2) => Double -> t
 decagon = regPoly 10
 
 -- | A regular hendecagon, with sides of the given length and base
 --   parallel to the x-axis.
 --
---   <<diagrams/hendecagonEx.svg#diagram=hendecagonEx&width=100>>
+--   <<diagrams/src_Diagrams_TwoD_Shapes_hendecagonEx.svg#diagram=hendecagonEx&width=100>>
 hendecagon :: (TrailLike t, V t ~ R2) => Double -> t
 hendecagon = regPoly 11
 
 -- | A regular dodecagon, with sides of the given length and base
 --   parallel to the x-axis.
 --
---   <<diagrams/dodecagonEx.svg#diagram=dodecagonEx&width=100>>
+--   <<diagrams/src_Diagrams_TwoD_Shapes_dodecagonEx.svg#diagram=dodecagonEx&width=100>>
 dodecagon :: (TrailLike t, V t ~ R2) => Double -> t
 dodecagon = regPoly 12
 
 ------------------------------------------------------------
 --  Other shapes  ------------------------------------------
 ------------------------------------------------------------
+data RoundedRectOpts = RoundedRectOpts { _radiusTL :: Double
+                                       , _radiusTR :: Double
+                                       , _radiusBL :: Double
+                                       , _radiusBR :: Double
+                                       }
+
+makeLenses ''RoundedRectOpts
+
+instance Default RoundedRectOpts where
+  def = RoundedRectOpts 0 0 0 0
 
 -- | @roundedRect w h r@ generates a closed trail, or closed path
 --   centered at the origin, of an axis-aligned rectangle with width
@@ -236,21 +255,21 @@ dodecagon = regPoly 12
 --   a different radius for each corner individually, use
 --   'roundedRect'' instead.
 --
---   <<diagrams/roundedRectEx.svg#diagram=roundedRectEx&width=400>>
+--   <<diagrams/src_Diagrams_TwoD_Shapes_roundedRectEx.svg#diagram=roundedRectEx&width=400>>
 --
---   > roundedRectEx = pad 1.1 . centerXY $ hcat' with { sep = 0.2 }
+--   > roundedRectEx = pad 1.1 . centerXY $ hcat' (with & sep .~ 0.2)
 --   >   [ roundedRect  0.5 0.4 0.1
 --   >   , roundedRect  0.5 0.4 (-0.1)
---   >   , roundedRect' 0.7 0.4 with { radiusTL = 0.2
---   >                               , radiusTR = -0.2
---   >                               , radiusBR = 0.1 }
+--   >   , roundedRect' 0.7 0.4 (with & radiusTL .~ 0.2
+--   >                                & radiusTR .~ -0.2
+--   >                                & radiusBR .~ 0.1)
 --   >   ]
 
 roundedRect :: (TrailLike t, V t ~ R2) => Double -> Double -> Double -> t
-roundedRect w h r = roundedRect' w h (with { radiusTL = r,
-                                             radiusBR = r,
-                                             radiusTR = r,
-                                             radiusBL = r})
+roundedRect w h r = roundedRect' w h (def & radiusTL .~ r
+                                          & radiusBR .~ r
+                                          & radiusTR .~ r
+                                          & radiusBL .~ r)
 
 -- | @roundedRect'@ works like @roundedRect@ but allows you to set the radius of
 --   each corner indivually, using @RoundedRectOpts@. The default corner radius is 0.
@@ -279,7 +298,7 @@ roundedRect' w h opts
         rBL                 = clampCnr radiusBR radiusTL radiusTR radiusBL
         rTR                 = clampCnr radiusTL radiusBR radiusBL radiusTR
         rBR                 = clampCnr radiusBL radiusTR radiusTL radiusBR
-        clampCnr rx ry ro r = let (rx',ry',ro',r') = (rx opts, ry opts, ro opts, r opts)
+        clampCnr rx ry ro r = let (rx',ry',ro',r') = (opts^.rx, opts^.ry, opts^.ro, opts^.r)
                                 in clampDiag ro' . clampAdj h ry' . clampAdj w rx' $ r'
         -- prevent curves of adjacent corners from overlapping
         clampAdj len adj r  = if abs r > len/2
@@ -293,12 +312,4 @@ roundedRect' w h opts
         mkCorner k r | r == 0    = mempty
                      | r < 0     = doArc 3 2
                      | otherwise = doArc 0 1
-                     where doArc d d' = arc' r ((k+d)/4) ((k+d')/4:: Turn)
-
-data RoundedRectOpts = RoundedRectOpts { radiusTL :: Double
-                                       , radiusTR :: Double
-                                       , radiusBL :: Double
-                                       , radiusBR :: Double
-                                       }
-instance Default RoundedRectOpts where
-  def = RoundedRectOpts 0 0 0 0
+                     where doArc d d' = arc' r ((k+d)/4 @@ turn) ((k+d')/4 @@ turn)
